package com.bocloud.ams.booter.controller.module;

import com.bocloud.ams.entity.model.ModuleTopo;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.service.module.ModuleRelationService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Tag(name = "模型关系管理")
@RestController
@RequestMapping("/v1/module/relation")
public class ModuleRelationController {

    @Autowired
    private ModuleRelationService moduleRelationService;

    @GetMapping(value = "/relevancy")
    @Operation(tags = {"CMC", "CSC"}, summary = "根据拓扑，模型数据查询模型关系")
    public GeneralResult<ModuleRelation> relevancy(@RequestParam("relationCategoryId") Long relationCategoryId,
                                                   @RequestParam("sourceId") Long sourceId,
                                                   @RequestParam("targetId") Long targetId,
                                                   @RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleRelationService.relevancy(relationCategoryId, sourceId, targetId, positionKey);
    }

    /**
     * 查询模型关系记录
     *
     * @param moduleId
     * @param relationCategoryId
     * @return
     */
    @GetMapping(value = "/topo")
    @Operation(tags = {"CMC", "CSC"}, summary = "根据模型ID和关联类型分组查询模型关系拓扑")
    public GeneralResult<ModuleTopo> list(@RequestParam("moduleId") Long moduleId,
                                          @RequestParam("relationCategoryId") Long relationCategoryId,
                                          @RequestParam(value = "positionKey", required = false) String positionKey) {
        if (null == moduleId || 0 == moduleId || null == relationCategoryId || 0 == relationCategoryId) {
            return ResultTools.formatErrResult();
        } else {
            return moduleRelationService.list(moduleId, relationCategoryId, positionKey);
        }
    }

    /**
     * 查询模型关系记录
     *
     * @param relationCategoryId
     * @return
     */
    @GetMapping(value = "/{relationCategoryId}/topo")
    @Operation(tags = {"CMC", "CSC"}, summary = "根据关联类型分组ID查询模型拓扑")
    public GeneralResult<ModuleTopo> listAll(@PathVariable(value = "relationCategoryId") Long relationCategoryId,
                                             @RequestParam(value = "positionKey", required = false) String positionKey) {

        if (null == relationCategoryId || 0 == relationCategoryId) {
            return ResultTools.formatErrResult();
        }

        return moduleRelationService.listByCategoryId(relationCategoryId, positionKey);

    }

    /**
     * 查询模型关系记录
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型关系详情")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = "positionKey", required = false) String positionKey) {

        if (null == id || 0 == id) {
            return ResultTools.formatErrResult();
        }

        return moduleRelationService.detail(id, positionKey);

    }

    /**
     * 新增模型关系
     *
     * @param record
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "模型关系新增")
    public GeneralResult<Void> save(@RequestBody ModuleRelation record,
        @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }
        return moduleRelationService.create(record, context.getTarget());

    }

    /**
     * 修改模型关系
     *
     * @param record
     * @param context
     * @return
     */

    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型关系修改")
    public GeneralResult modify(@RequestBody ModuleRelation record, @PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }
        record.setId(id);
        return moduleRelationService.modify(record, context.getTarget());

    }

    /**
     * 移除模型关系
     *
     * @param id
     * @param context
     * @return
     */

    @DeleteMapping("/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型关系删除")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleRelationService.remove(id, context.getTarget());

    }

    /**
     * 关联类型分组id查询模型集合
     *
     * @param relationCategoryId
     * @return
     */
    @GetMapping(value = "/{relationCategoryId}/module")
    @Operation(tags = {"CMC", "CSC"}, summary = "关联类型分组id查询模型集合")
    public GeneralResult<List<CiModule>> moduleList(@PathVariable(value = "relationCategoryId") Long relationCategoryId,
                                                    @RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleRelationService.moduleList(relationCategoryId, positionKey);
    }

    private boolean paramsCheck(ModuleRelation record) {
        // 参数校验
        if (null == record || null == record.getSourceId() || 0 == record.getSourceId() || null == record.getTargetId()
            || 0 == record.getTargetId() || null == record.getRelationCategoryId()
            || 0 == record.getRelationCategoryId() || null == record.getRelationId() || 0 == record.getRelationId()) {
            return false;
        }
        if (StringUtils.isEmpty(record.getRule())) {
            return false;
        }
        String type = record.getRule();
        String[] types = type.split("-");
        record.setSourceType(types[0]);
        record.setTargetType(types[1]);
        return true;
    }

    @GetMapping(value = "/instances")
    @Operation(tags = {"CMC"}, summary = "模型关系实例列表")
    public GeneralResult instances(Pager pager,
                                   @Value(Common.REQ_CONTEXT) RequestContext context,
                                   @RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleRelationService.instances(pager, context.getTarget(), positionKey);

    }

}
