package com.bocloud.ams.service.association.impl;

import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.association.AutoRelationRuleItem;
import com.bocloud.ams.entity.association.AutoRelationRuleItemValue;
import com.bocloud.ams.repository.association.AutoRelationRuleItemRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleItemValueRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleRepository;
import com.bocloud.ams.service.association.AutoRelationRuleService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/9 0009 16:48
 * @Description:
 */
@Slf4j
@Service("autoRelationRuleService")
public class AutoRelationRuleServiceImpl implements AutoRelationRuleService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private AutoRelationRuleRepository autoRelationRuleRepository;
    @Autowired
    private AutoRelationRuleItemRepository autoRelationRuleItemRepository;
    @Autowired
    private AutoRelationRuleItemValueRepository autoRelationRuleItemValueRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<AutoRelationRule>> list(Pager pager, String positionKey) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.autoRelationRuleRepository.count(pager.getParams(), positionKey);
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.autoRelationRuleRepository.list(pager.getParams(), pager.getSorter(), positionKey);
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                List<AutoRelationRule> list = this.autoRelationRuleRepository.list(pager, positionKey);
                for (AutoRelationRule rule : list) {
                    rule.setCreator(redisTemplate.opsForValue().get("User_Name_" + rule.getCreatorId()));
                    List<AutoRelationRuleItem> items = autoRelationRuleItemRepository.queryByRuleId(rule.getId(), positionKey);
                    for (AutoRelationRuleItem item : items) {
                        List<AutoRelationRuleItemValue> values =
                            autoRelationRuleItemValueRepository.queryByItemId(item.getId(), positionKey);
                        item.setValues(values);
                    }
                    rule.setItems(items);
                }
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<AutoRelationRule>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query notice rule list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> save(AutoRelationRule rule, Long target) {
        try {
            if (autoRelationRuleRepository.countSameName(rule.getName()) > 0) {
                return new GeneralResult<>(false, "添加失败，该名称已存在，请尝试其他名称");
            }
            if (autoRelationRuleRepository.isOnly(rule.getRelationCategoryId(), rule.getSourceModuleId(),
                rule.getTargetModuleId()) > 0) {
                return new GeneralResult<>(false, "同一拓扑下，两个模型之间只允许存在一种关联规则，请检查");
            }
            rule.setCreatorId(target);
            rule.setGmtCreate(new Date());
            autoRelationRuleRepository.save(rule);
            rule.getItems().forEach(item -> {
                item.setRelationRuleId(rule.getId());
                item.setCreatorId(target);
                item.setGmtCreate(new Date());
                autoRelationRuleItemRepository.save(item);
                List<AutoRelationRuleItemValue> itemValues = item.getValues();
                if (null != itemValues) {
                    itemValues.forEach(value -> {
                        value.setRelationRuleId(rule.getId());
                        value.setItemId(item.getId());
                        value.setCreatorId(target);
                        value.setGmtCreate(new Date());
                        autoRelationRuleItemValueRepository.save(value);
                    });
                }
            });
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("create rule is fail:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> modify(Long id, AutoRelationRule rule, Long target) {
        String path = AutoRelationRule.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            AutoRelationRule relationRule = autoRelationRuleRepository.query(id);
            if (null == relationRule) {
                return new GeneralResult<>(false, "数据不存在");
            }
            rule.setGmtModify(new Date());
            rule.setMenderId(target);
            autoRelationRuleRepository.update(rule);
            // 更新规则明细数据
            autoRelationRuleItemRepository.deleteByRuleId(id, target);
            autoRelationRuleItemValueRepository.deleteByRuleId(id, target);
            rule.getItems().forEach(item -> {
                item.setId(null);
                item.setRelationRuleId(rule.getId());
                item.setCreatorId(target);
                item.setGmtCreate(new Date());
                autoRelationRuleItemRepository.save(item);
                List<AutoRelationRuleItemValue> itemValues = item.getValues();
                if (null != itemValues) {
                    itemValues.forEach(value -> {
                        value.setId(null);
                        value.setRelationRuleId(rule.getId());
                        value.setItemId(item.getId());
                        value.setCreatorId(target);
                        value.setGmtCreate(new Date());
                        autoRelationRuleItemValueRepository.save(value);
                    });
                }
            });
            return new GeneralResult<>(true, "更新成功");
        } catch (Exception e) {
            log.error("modify rule is fail:", e);
            return new GeneralResult<>(false, "更新失败");
        }
    }

    @Override
    public GeneralResult<AutoRelationRule> detail(Long id, String positionKey) {
        try {
            AutoRelationRule relationRule = autoRelationRuleRepository.query(id, positionKey);
            if (null == relationRule) {
                return new GeneralResult<>(false, "数据不存在");
            }
            // 查询规则明细数据
            List<AutoRelationRuleItem> items = autoRelationRuleItemRepository.queryByRuleId(id, positionKey);
            for (AutoRelationRuleItem item : items) {
                List<AutoRelationRuleItemValue> values =
                    autoRelationRuleItemValueRepository.queryByItemId(item.getId(), positionKey);
                item.setValues(values);
            }
            relationRule.setItems(items);
            return new GeneralResult<>(true, relationRule, "查询成功");
        } catch (Exception e) {
            log.error("query rule detail is fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> remove(Long id, Long target) {
        String path = AutoRelationRule.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            AutoRelationRule relationRule = autoRelationRuleRepository.query(id);
            if (null == relationRule) {
                return new GeneralResult<>(false, "数据不存在");
            }
            autoRelationRuleRepository.delete(id, target);
            // 删除关联数据
            autoRelationRuleItemRepository.deleteByRuleId(id, target);
            autoRelationRuleItemValueRepository.deleteByRuleId(id, target);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("delete rule is fail:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<AutoRelationRuleItem> itemDetail(Long itemId, String positionKey) {
        try {
            AutoRelationRuleItem ruleItem = autoRelationRuleItemRepository.query(itemId, positionKey);
            if (null == ruleItem) {
                return new GeneralResult<>(false, "规则明细数据不存在");
            }
            List<AutoRelationRuleItemValue> itemValues = autoRelationRuleItemValueRepository.queryByItemId(itemId, positionKey);
            ruleItem.setValues(itemValues);
            return new GeneralResult<>(true, ruleItem, "查询成功");
        } catch (Exception e) {
            log.error("query rule item is fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<AutoRelationRule>> list(Long moduleId, Long categoryId) {
        try {
            List<AutoRelationRule> list = autoRelationRuleRepository.queryByModuleIdAndCategoryId(moduleId, categoryId);
            return new GeneralResult<>(true, list, "查询成功");
        } catch (Exception e) {
            log.error("Query rule list by moduleId and relationCategoryId fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

}
