package com.bocloud.ams.service.config.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.config.CmdbConfig;
import com.bocloud.ams.repository.config.CmdbConfigRepository;
import com.bocloud.ams.service.config.CmdbConfigService;
import com.megatron.common.exception.InternalServerException;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service("cmdbConfigService")
public class CmdbConfigServiceImpl implements CmdbConfigService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CmdbConfigRepository cmdbConfigRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * 修改系统配置
     *
     * @param config
     *            系统配置对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result modify(CmdbConfig config) throws InternalServerException {
        String path = CmdbConfig.class.getSimpleName() + "_" + config.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CmdbConfig object = this.cmdbConfigRepository.query(config.getId());
            Assert.notNull(object, "CmdbConfig does not exist!");
            BeanUtils.copyProperties(config, object);
            object.setGmtModify(new Date());
            this.cmdbConfigRepository.modify(object);
            redisTemplate.opsForValue().set(object.getCode(), object.getValue());
            return new Result(true, "修改成功");
        } catch (Exception e) {
            log.error("Modify CmdbConfig fail:", e);
            throw new InternalServerException("修改失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 批量更新系统配置
     *
     * @param models
     *            系统配置集合
     * @return
     */
    @Override
    public Result batchUpdate(List<CmdbConfig> models, RequestContext context) {
        String[] titles = new String[] {"id", "value", "gmtModify", "menderId"};
        try {
            Map<String, String> configMap = new HashMap<>(8);
            for (CmdbConfig model : models) {
                configMap.put(model.getCode(), model.getValue());
            }
            Set<String> configCodes = configMap.keySet();
            List<CmdbConfig> list = cmdbConfigRepository.listByCodes(configCodes);
            for (CmdbConfig config : list) {
                config.setValue(configMap.get(config.getCode()));
                config.setMenderId(context.getTarget());
                config.setGmtModify(new Date());
            }
            boolean flag = cmdbConfigRepository.batchUpdate(list, titles);
            if (!flag) {
                return new GeneralResult<>(false, "修改失败");
            }
            for (CmdbConfig config : list) {
                redisTemplate.opsForValue().set(config.getCode(), config.getValue());
            }
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("batch modify  system config  fail:", e);
            return new GeneralResult<>(false, "修改失败" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<Map<String, String>> listConfigs(String category, List<String> codes, String positionKey) {
        try {
            List<CmdbConfig> configs;
            if (StringUtils.hasText(category)) {
                configs = cmdbConfigRepository.listByCategory(category, positionKey);
            } else {
                configs = cmdbConfigRepository.listByCodes(new HashSet<>(codes), positionKey);
            }
            Map<String, CmdbConfig> configMap =
                configs.stream().collect(Collectors.toMap(CmdbConfig::getCode, Function.identity()));
            Map<String, String> result = new HashMap<>(8);
            for (Map.Entry<String, CmdbConfig> entry : configMap.entrySet()) {
                String value = entry.getValue().getValue();
                result.put(entry.getKey(), value);
            }
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("list configs by codes error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<JSONArray> listCategoryTree(String category, String positionKey) {
        try {
            List<CmdbConfig> configs = cmdbConfigRepository.listByCategory(category, positionKey);
            Map<String, List<CmdbConfig>> configMap =
                configs.stream().collect(Collectors.groupingBy(CmdbConfig::getSubclass));
            JSONArray jsonArray = new JSONArray();
            for (Map.Entry<String, List<CmdbConfig>> entry : configMap.entrySet()) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", entry.getKey());
                jsonObject.put("values", entry.getValue());
                jsonArray.add(jsonObject);
            }
            return new GeneralResult<>(true, jsonArray, "查询成功");
        } catch (Exception e) {
            log.error("list configs tree by category error,", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

}
