package com.bocloud.ams.task.executor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.task.exception.NodeFormatException;
import com.bocloud.ams.task.model.HostResult;
import com.bocloud.ams.task.model.Module;
import com.bocloud.ams.task.model.WorkerHost;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Binding.DestinationType;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;

/**
 * 远端配置任务执行线程，用来支持任务执行能分发的对某个worker的请求操作
 *
 * @author dmw
 */
@Slf4j
public class RemoteWorker implements Callable<GeneralResult<?>> {
    private static final String EXCHANGE = BoCloudExchange.Default.DIRECT.getExchange();
    private static final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");
    private static final Integer DEFAULT_TIMEOUT = 1000 * 60 * 10;
    private static final String QUEUE = "queue";
    private static final String TIMEOUT = "timeout";
    private List<WorkerHost> targets;

    // 默认超时时间是10分钟
    private Integer timeout;
    // worker返回任务结果的队列
    private String callback;
    private boolean sync = false;
    private RemoteService service;
    private RabbitAdmin rabbitAdmin;

    /**
     * 构造函数
     *
     * @param service     worker服务实例
     * @param rabbitAdmin RabbitMQ管理实例
     * @param timeout     超时时间 单位为S
     * @throws NodeFormatException
     */
    public RemoteWorker(RemoteService service, RabbitAdmin rabbitAdmin, Integer timeout, String callback, boolean sync)
            throws NodeFormatException {
        this.service = service;
        this.rabbitAdmin = rabbitAdmin;
        if (null == timeout) {
            this.timeout = DEFAULT_TIMEOUT;
        } else {
            this.timeout = timeout * 1000;
        }
        this.callback = callback;
        this.sync = sync;
        init();
    }

    @Override
    public GeneralResult<?> call() throws Exception {
        if (null == service || !service.isExist()) {
            log.error("remote worker does not exist!");
            return handleError("远程Worker服务不存在");
        }
        if (sync) {
            return handleSync();
        } else {
            return handleAsync();
        }
    }

    /**
     * 处理同步调用
     *
     * @return
     */
    @SuppressWarnings("unchecked")
    private GeneralResult<?> handleSync() {
        log.info("start to sync call remote worker [{}] and timeout is :{} ms ", service.getUrl(), timeout);
        Map<String, Object> params = (Map<String, Object>) service.getParam().getParams();
        if (null == params) {
            params = new HashMap<String, Object>();
        }
        params.put("timeout", this.timeout / 1000);
        service.getParam().setParams(params);
        GeneralResult<?> invoke = service.invoke();
        if (log.isDebugEnabled()) {
            log.debug("remote worker response is {}", invoke);
        }
        return handleResult(invoke);
    }

    /**
     * 处理异步调用
     *
     * @return
     */
    @SuppressWarnings("unchecked")
    private GeneralResult<?> handleAsync() {
        if (StringUtils.isEmpty(callback)) {
            this.callback = IDFactory.instance().uuid();
        }
        GeneralResult<?> result;
        Map<String, Object> params = (Map<String, Object>) service.getParam().getParams();
        if (null == params) {
            params = new HashMap<String, Object>();
        }
        Object object = params.get("module");
        if (null == object) {
            log.info("start to async call remote worker [{}] with callback queue is [{}] and timeout is :{}ms",
                    service.getUrl(), this.callback, timeout);
        } else {
            Module<?> module = (Module<?>) object;
            log.info(
                    "start to async call remote worker [{}] with callback queue is [{}] and task module is [{}] and timeout is :{}ms",
                    service.getUrl(), this.callback, module.getName(), timeout);
        }
        try {
            params.put(QUEUE, this.callback);
            service.getParam().setParams(params);
            // 声明worker结果的存放队列并将其绑定到指定的exchange:bocloud.direct.exchange
            rabbitAdmin.declareQueue(new Queue(this.callback, false, false, true, arguments));
            rabbitAdmin
                    .declareBinding(new Binding(this.callback, DestinationType.QUEUE, EXCHANGE, this.callback, arguments));
            log.info("request params is {}", JSONObject.toJSON(service.getParam().getParams()));
            GeneralResult<?> invoke = service.invoke();
            if (invoke.isFailed()) {
                log.error("invoke remote worker [{}] failed for : {}", service.getUrl(), invoke.getMessage());
                return handleError(invoke.getMessage());
            }
            // 这里说明请求成功发送到worker上，接下来可以等待结果消息了
            Message message = rabbitAdmin.getRabbitTemplate().receive(callback, timeout + 60 * 1000);
            if (null == message) {
                log.error("get callback from remote worker by queue [{}] timeout", callback);
                result = handleError("获取任务结果消息超时！");
            } else {
                String content = new String(message.getBody(), "UTF-8");
                log.info("get result [{}] from remote worker by queue [{}]", content, callback);
                if (log.isDebugEnabled()) {
                    log.debug("callback from remote worker is :\n{}", content);
                }
                try {
                    result = handleResult(JSONObject.parseObject(content, GeneralResult.class));
                } catch (Exception e) {
                    log.error("callback data transfer exception!", e);
                    result = handleError("返回数据格式异常!");
                }
            }
        } catch (Exception e) {
            log.error("等待Worker 【{}】返回任务结果消息异常:", service.getUrl(), e);
            result = handleError(e.getMessage());
        } finally {
            rabbitAdmin.deleteQueue(this.callback);
        }
        if (log.isDebugEnabled()) {
            log.debug("执行结果: {}", result);
        }
        return result;
    }

    /**
     * 构建消息
     *
     * @param invoke 远程调用结果
     * @return
     * @throws NodeFormatException
     */
    private GeneralResult<?> handleResult(GeneralResult<?> invoke) {
        if (invoke.isSuccess()) {
            return invoke;
        }
        if (null == invoke.getData()) {
            return handleError(invoke.getMessage());
        } else {
            return invoke;
        }
    }

    private GeneralResult<?> handleError(String message) {
        List<HostResult> hostResults = new ArrayList<>();
        for (WorkerHost target : targets) {
            hostResults.add(new HostResult(false, message, target.getHost(), target.getRawName(), target.getPort(),
                    target.getUser()));
        }
        return new GeneralResult<>(false, hostResults, message);
    }

    @SuppressWarnings("unchecked")
    private void init() throws NodeFormatException {
        Map<String, Object> params = (Map<String, Object>) service.getParam().getParams();
        if (null == params) {
            params = new HashMap<String, Object>();
        }
        params.put(TIMEOUT, timeout / 1000);
        params.put(QUEUE, this.callback);
        service.getParam().setParams(params);
        service.setDataFormat(PostDataFormat.RAW);
        String targetParam = JSONObject.toJSONString(params.get("targets"));
        targets = WorkerHost.parseTargets(targetParam);
        if (targets.isEmpty()) {
            throw new NodeFormatException("目标机器参数信息格式错误");
        }
    }
}
