package com.bocloud.ams.task.executor;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletionService;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.springframework.amqp.rabbit.core.RabbitAdmin;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.task.exception.NodeFormatException;
import com.bocloud.ams.task.model.HostResult;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

import lombok.extern.slf4j.Slf4j;

/**
 * 远程Worker调用任务分发器，用来对worker请求进行参数校验和请求分片调度。
 *
 * @author dmw
 */
@Slf4j
public class WorkerDispatcher {
    private static final String URL = "/api/gaia/v1/job/submit";
    private static final Service SERVICE = Service.create(BoCloudService.GAIA.getName());
    private Map<String, Object> headers = MapTools.simpleMap("Content-Type", "application/json");
    private ExecutorService executor = null;
    private CompletionService<GeneralResult<?>> completionService = null;

    private ServiceFactory serviceFactory;
    private RabbitAdmin rabbitAdmin;

    /**
     *
     */
    public WorkerDispatcher() {
        super();
    }

    /**
     * 构造方法
     *
     * @param serviceFactory
     * @param rabbitAdmin
     */
    public WorkerDispatcher(ServiceFactory serviceFactory, RabbitAdmin rabbitAdmin) {
        this.serviceFactory = serviceFactory;
        this.rabbitAdmin = rabbitAdmin;
    }

    /**
     * worker任务分发执行
     *
     * @return
     * @throws NodeFormatException
     */
    public GeneralResult<?> execute(List<Map<String, Object>> params, Integer timeout) throws NodeFormatException {
        log.info("start to execute multiful remote worker dispatch ...");
        if (null == params || params.isEmpty()) {
            log.error("remote worker param is empty!");
            throw new NodeFormatException("worker param is illegel!");
        }
        List<HostResult> hostResults = new ArrayList<>();
        int corePoolSize = params.size() + 1;
        executor = new ThreadPoolExecutor(corePoolSize, corePoolSize, 30, TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(corePoolSize), (Runnable runnable) -> {
                return new Thread(runnable);
            });
        completionService = new ExecutorCompletionService<>(executor);
        if (serviceFactory.exist(SERVICE).isFailed()) {
            log.error("remote worker does not exist!");
            return GeneralResult.FAILED("远程Worker不存在!");
        }
        for (Map<String, Object> param : params) {
            RemoteService service = serviceFactory.build(SERVICE, URL, HttpMethod.OPERATE, headers, param);
            RemoteWorker task = new RemoteWorker(service, rabbitAdmin, timeout, null, false);
            completionService.submit(task);
        }
        int finished = 0;
        int workerNum = params.size();
        // 处理任务执行的结果
        GeneralResult<?> GeneralResult = null;
        boolean hasError = false;
        while (finished < workerNum) {
            try {
                GeneralResult = completionService.take().get();
                if (GeneralResult.isFailed()) {
                    // 单次任务请求处理失败！
                    log.error("Worker任务执行失败！{}", GeneralResult.getMessage());
                    hasError = true;
                }
                hostResults.addAll(buildHostResult(GeneralResult));
            } catch (Exception e) {
                log.error("单次请求worker任务执行异常", e);
                hasError = true;
            }
            finished++;
        }
        // TODO 这里需要处理失败重试机制，后续优化
        if (null != executor) {
            executor.shutdown();
        }
        if (hasError) {
            log.error("task of execute failed!");
            if (ListTool.isEmpty(hostResults)) {
                hostResults.addAll(buildHostResult(params, false, "任务执行失败：可能所设定时间已经超时"));
            }
            return new GeneralResult<>(false, hostResults, "任务执行失败：可能所设定时间已经超时");
        } else {
            // 没有失败任务，直接返回处理结果
            log.info("task of execute success!");
            return new GeneralResult<>(true, hostResults, "任务执行成功！");
        }
    }

    private List<HostResult> buildHostResult(GeneralResult<?> result) {
        List<HostResult> hostResults = new ArrayList<>();
        if (null != result.getData()) {
            try {
                hostResults = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
            } catch (Exception e) {
                log.error("build host result error!", e);
            }
        }
        return hostResults;
    }

    private List<HostResult> buildHostResult(List<Map<String, Object>> params, Boolean success, String message) {
        List<HostResult> hostResults = new ArrayList<>();
        for (Map<String, Object> param : params) {
            if (null == param.get("targets")) {
                continue;
            }
            try {
                List<HostResult> results =
                    JSONArray.parseArray(JSONObject.toJSONString(param.get("targets")), HostResult.class);
                for (HostResult result : results) {
                    result.setSuccess(success);
                    result.setMessage(message);
                }
                hostResults.addAll(results);
            } catch (Exception e) {
                log.error("build host result error!", e);
            }
        }
        return hostResults;
    }

}
